## TODO: Save obs percentiles
## TODO: Insert vardim to simplify the code?

source("modules/Saving/R/get_dir.R")
source("modules/Saving/R/get_filename.R")
source("modules/Saving/R/Utils.R")
source("modules/Saving/R/save_forecast.R")
source("modules/Saving/R/save_observations.R")
source("modules/Saving/R/save_metrics.R")
source("modules/Saving/R/save_corr.R")
source("modules/Saving/R/save_probabilities.R")
source("modules/Saving/R/save_percentiles.R")

save_data <- function(recipe, data,
                      skill_metrics = NULL,
                      probabilities = NULL) {
  # Wrapper for the saving functions.
  # recipe: The auto-s2s recipe
  # archive: The auto-s2s archive
  # data: output of load_datasets()
  # data: output of calibrate_datasets()
  # skill_metrics: output of compute_skill_metrics()
  # probabilities: output of compute_probabilities()
  # mean_bias: output of compute_mean_bias()

  # Sanity checks
  if (is.null(recipe)) {
    error(recipe$Run$logger, "The 'recipe' parameter is mandatory.")
    stop()
  }

  if (is.null(data)) {
    error(recipe$Run$logger,
          paste("The 'data' parameter is mandatory. It should be a list",
                "of at least two s2dv_cubes containing the hcst and obs."))
    stop()
  }
  # Create output directory
  outdir <- get_dir(recipe)
  for (directory in outdir) {
    dir.create(directory, showWarnings = FALSE, recursive = TRUE)
  }

  # Separate ensemble correlation from the rest of the metrics, as it has one
  # extra dimension "ensemble" and must be saved to a different file
  if ("corr" %in% names(skill_metrics)) {
    corr_metric_names <- grep("^corr", names(skill_metrics))
    corr_metrics <- skill_metrics[corr_metric_names]
    skill_metrics <- skill_metrics[-corr_metric_names]
    if (length(skill_metrics) == 0) {
      skill_metrics <- NULL
    }
  } else {
    corr_metrics <- NULL
  }

  # Iterate over variables to subset s2dv_cubes and save outputs
  for (var in 1:data$hcst$dims[['var']]) {
    info(recipe$Run$logger,
         paste("Saving outputs for", data$hcst$attrs$Variable$varName[var]))
    # Export hindcast, forecast and observations
    subset_hcst <- CST_Subset(data$hcst, along = 'var', indices = var,
                              drop = F, var_dim = 'var', dat_dim = 'dat')

    save_forecast(recipe = recipe,
                  data_cube = subset_hcst,
                  outdir = outdir[var],
                  type = 'hcst')
    if (!is.null(data$fcst)) {
      subset_fcst <- CST_Subset(data$fcst, along = 'var', indices = var,
                                drop = F, var_dim = 'var', dat_dim = 'dat')
      save_forecast(recipe = recipe, 
                    data_cube = subset_fcst,
                    outdir = outdir[var],
                    type = 'fcst')
    }
    subset_obs <- CST_Subset(data$obs, along = 'var', indices = var,
                             drop = F, var_dim = 'var', dat_dim = 'dat')
    save_observations(recipe = recipe, 
                      data_cube = subset_obs,
                      outdir = outdir[var])

    # Export skill metrics
    if (!is.null(skill_metrics)) {
      if (data$hcst$dims[['var']] == 1) {
        save_metrics(recipe = recipe,
                     skill = skill_metrics,
                     data_cube = data$hcst,
                     outdir = outdir[var])
        if (!is.null(corr_metrics)) {
          save_corr(recipe = recipe,
                    skill = corr_metrics, 
                    data_cube = data$hcst,
                    outdir = outdir[var])
        }
      } else {
        subset_skill <- lapply(skill_metrics, function(x) {
                               ClimProjDiags::Subset(x, along = 'var',
                                                     indices = var,
                                                     drop = 'selected')})
        save_metrics(recipe = recipe,
                     skill = subset_skill,
                     data_cube = subset_hcst,
                     outdir = outdir[var])
        if (!is.null(corr_metrics)) {
          subset_corr <- lapply(corr_metrics, function(x) {
                                ClimProjDiags::Subset(x, along = 'var',
                                                      indices = var,
                                                      drop = 'selected')})
          save_corr(recipe = recipe, 
                    skill = subset_corr,
                    data_cube = subset_hcst,
                    outdir = outdir[var])
        }
      }
    }
  
    # Export probabilities onto outfile
    if (!is.null(probabilities)) {
      if (data$hcst$dims[['var']] == 1) {
        save_percentiles(recipe = recipe,
                         percentiles = probabilities$percentiles, 
                         data_cube = data$hcst,
                         outdir = outdir[var])
        save_probabilities(recipe = recipe,
                           probs = probabilities$probs,
                           data_cube = data$hcst,
                           outdir = outdir[var],
                           type = "hcst")
        if (!is.null(probabilities$probs_fcst)) {
          save_probabilities(recipe = recipe,
                             probs = probabilities$probs_fcst,
                             data_cube = subset_fcst,
                             outdir = outdir[var],
                             type = "fcst")
        }
      } else {
        subset_percentiles <- lapply(probabilities$percentiles, function(x) {
                                     ClimProjDiags::Subset(x, along = 'var',
                                                           indices = var,
                                                           drop = 'selected')})
        save_percentiles(recipe = recipe,
                         percentiles = subset_percentiles,
                         data_cube = subset_hcst,
                         outdir = outdir[var])
        subset_probs <- lapply(probabilities$probs, function(x) {
                               ClimProjDiags::Subset(x, along = 'var',
                                                     indices = var,
                                                     drop = 'selected')})
        save_probabilities(recipe = recipe,
                           probs = subset_probs,
                           data_cube = subset_hcst,
                           outdir = outdir[var], 
                           type = "hcst")
        if (!is.null(probabilities$probs_fcst)) {
          subset_probs_fcst <- lapply(probabilities$probs_fcst, function(x) {
                                      ClimProjDiags::Subset(x, along = 'var',
                                                            indices = var,
                                                            drop = 'selected')})
          save_probabilities(recipe = recipe,
                             probs = subset_probs_fcst,
                             data_cube = subset_fcst,
                             outdir = outdir[var],
                             type = "fcst")
        }
      }
    }
  }
}
