plot_skill_metrics <- function(recipe, data_cube, skill_metrics,
                               outdir, significance = F) {
  # recipe: Auto-S2S recipe
  # archive: Auto-S2S archive
  # data_cube: s2dv_cube object with the corresponding hindcast data
  # skill_metrics: list of named skill metrics arrays
  # outdir: output directory
  # significance: T/F, whether to display the significance dots in the plots

  ## TODO: OPTION for CERISE: Using PuOr
  # Abort if frequency is daily
  if (recipe$Analysis$Variables$freq == "daily_mean") {
    error(recipe$Run$logger, "Visualization functions not yet implemented
          for daily data.")
    stop()
  }
  # Abort if skill_metrics is not list
  if (!is.list(skill_metrics) || is.null(names(skill_metrics))) {
    stop("The element 'skill_metrics' must be a list of named arrays.")
  }
  
  latitude <- data_cube$coords$lat
  longitude <- data_cube$coords$lon
  archive <- get_archive(recipe)
  system_name <- archive$System[[recipe$Analysis$Datasets$System$name]]$name
  hcst_period <- paste0(recipe$Analysis$Time$hcst_start, "-",
                        recipe$Analysis$Time$hcst_end)
  init_month <- as.numeric(substr(recipe$Analysis$Time$sdate,
                                  start = 1, stop = 2))
  month_label <- tolower(month.name[init_month])
  month_abbreviation <- month.abb[init_month]
  
  # Define color palette and number of breaks according to output format
  ## TODO: Make separate function
  if (tolower(recipe$Analysis$Output_format) %in% c("scorecards", "cerise")) {
    diverging_palette <- "purpleorange"
    sequential_palette <- "Oranges"
  } else {
    diverging_palette <- "bluered"
    sequential_palette <- "Reds"
  }
  
  # Group different metrics by type
  skill_scores <- c("rpss", "bss90", "bss10", "frpss", "crpss", "mean_bias_ss",
                    "enscorr", "rpss_specs", "bss90_specs", "bss10_specs",
                    "enscorr_specs", "rmsss")
  scores <- c("rps", "frps", "crps", "frps_specs")
  # Assign colorbar to each metric type
  ## TODO: Triangle ends
  for (name in c(skill_scores, scores, "mean_bias", "enssprerr")) {
    if (name %in% names(skill_metrics)) {
      # Define plot characteristics and metric name to display in plot
      if (name %in% c("rpss", "bss90", "bss10", "frpss", "crpss",
                      "rpss_specs", "bss90_specs", "bss10_specs",
                      "rmsss")) {
        display_name <- toupper(strsplit(name, "_")[[1]][1])
        skill <- skill_metrics[[name]]
        brks <- seq(-1, 1, by = 0.2)
        colorbar <- clim.colors(length(brks) + 1, diverging_palette)
        cols <- colorbar[2:(length(colorbar) - 1)]
        col_inf <- colorbar[1]
        col_sup <- NULL
      } else if (name == "mean_bias_ss") {
        display_name <- "Mean Bias Skill Score"
        skill <- skill_metrics[[name]]
        brks <- seq(-1, 1, by = 0.2)
        colorbar <- clim.colors(length(brks) + 1, diverging_palette)
        cols <- colorbar[2:(length(colorbar) - 1)]
        col_inf <- colorbar[1]
        col_sup <- NULL
      } else if (name %in% c("enscorr", "enscorr_specs")) {
        display_name <- "Ensemble Mean Correlation"
        skill <- skill_metrics[[name]]
        brks <- seq(-1, 1, by = 0.2)
        cols <- clim.colors(length(brks) - 1, diverging_palette)
        col_inf <- NULL
        col_sup <- NULL
      } else if (name %in% scores) {
        skill <- skill_metrics[[name]]
        display_name <- toupper(strsplit(name, "_")[[1]][1])
        brks <- seq(0, 1, by = 0.1)
        colorbar <- grDevices::hcl.colors(length(brks), sequential_palette)
        cols <- colorbar[1:(length(colorbar) - 1)]
        col_inf <- NULL
        col_sup <- colorbar[length(colorbar)]
      } else if (name == "enssprerr") {
        skill <- skill_metrics[[name]]
        display_name <- "Spread-to-Error Ratio"
        brks <- c(0, 0.6, 0.7, 0.8, 0.9, 1, 1.2, 1.4, 1.6, 1.8, 2)
        colorbar <- clim.colors(length(brks), diverging_palette)
        cols <- colorbar[1:length(colorbar) - 1]
        col_inf <- NULL
        col_sup <- colorbar[length(colorbar)]
      } else if (name == "mean_bias") {
        skill <- skill_metrics[[name]]
        display_name <- "Mean Bias"
        max_value <- max(abs(quantile(skill, 0.02, na.rm = T)),
                         abs(quantile(skill, 0.98, na.rm = T)))
        brks <- max_value * seq(-1, 1, by = 0.2)
        colorbar <- clim.colors(length(brks) + 1, diverging_palette)
        cols <- colorbar[2:(length(colorbar) - 1)]
        col_inf <- colorbar[1]
        col_sup <- colorbar[length(colorbar)]
      }
      options(bitmapType = "cairo")
      # Reorder dimensions
      skill <- Reorder(skill, c("time", "longitude", "latitude"))
      # If the significance has been requested and the variable has it,
      # retrieve it and reorder its dimensions. 
      significance_name <- paste0(name, "_significance")
      if ((significance) && (significance_name %in% names(skill_metrics))) {
        skill_significance <- skill_metrics[[significance_name]]
        skill_significance <- Reorder(skill_significance, c("time", 
                                                            "longitude",
                                                            "latitude"))
        # Split skill significance into list of lists, along the time dimension
        # This allows for plotting the significance dots correctly.
        skill_significance <- ClimProjDiags::ArrayToList(skill_significance,
                                                         dim = 'time',
                                                         level = "sublist",
                                                         names = "dots")
      } else {
        skill_significance <- NULL
      }
      # Define output file name and titles
      if (tolower(recipe$Analysis$Horizon) == "seasonal") {
        outfile <- paste0(outdir, name, "-", month_label, ".png")
      } else {
        outfile <- paste0(outdir, name, ".png")
      }
      toptitle <- paste(display_name, "-", data_cube$attrs$Variable$varName,
                        "-", system_name, "-", month_abbreviation,
                        hcst_period)
      months <- unique(lubridate::month(data_cube$attrs$Dates,
                                        label = T, abb = F))
      titles <- as.vector(months)
      # Plot
      suppressWarnings(
      PlotLayout(PlotEquiMap, c('longitude', 'latitude'),
                 asplit(skill, MARGIN=1), # Splitting array into a list
                 longitude, latitude,
                 special_args = skill_significance, 
                 dot_symbol = 20,
                 toptitle = toptitle,
                 title_scale = 0.6,
                 titles = titles,
                 filled.continents=F,
                 brks = brks,
                 cols = cols,
                 col_inf = col_inf,
                 col_sup = col_sup,
                 fileout = outfile,
                 bar_label_digits = 3,
                 bar_extra_margin = rep(0.9, 4),
                 bar_label_scale = 1.5,
                 axes_label_scale = 1.3)
      )
    }
  }
  info(recipe$Run$logger,
       "##### SKILL METRIC PLOTS SAVED TO OUTPUT DIRECTORY #####")
}
